@@ -65,7 +65,7 @@ class MapScene(QGraphicsScene):
     """
 
     def __init__(self, parent: "MapWidget" = None) -> None:
-        super().__init__(parent)
+        super(MapScene, self).__init__(parent)
         self.setItemIndexMethod(QGraphicsScene.ItemIndexMethod.BspTreeIndex)  # For efficient item indexing
 
     def mousePressEvent(self, event: QGraphicsSceneMouseEvent) -> None:
@@ -74,7 +74,7 @@ class MapScene(QGraphicsScene):
         :param event:
         :return:
         """
-        print(f"Scene pressed at {event.scenePos()}")
+        # print(f"Scene pressed at {event.scenePos()}")
         super().mousePressEvent(event)
 
     def mouseReleaseEvent(self, event):
@@ -100,30 +100,20 @@ class MapView(QGraphicsView):
     MapView
     """
 
-    def __init__(self, _scene: QGraphicsScene,
-                 map_widget: "MapWidget",
-                 start_level: int,
-                 startLat: float,
-                 startLon: float):
+    def __init__(self,
+                 scene: QGraphicsScene,
+                 map_widget: "MapWidget"):
         """
 
-        :param _scene:
+        :param scene:
         :param map_widget:
-        :param start_level:
-        :param startLat:
-        :param startLon:
         """
-        QGraphicsView.__init__(self, _scene)
+        QGraphicsView.__init__(self, scene)
 
-        self._scene = _scene
+        self._scene = scene
 
         self.map_widget = map_widget
 
-        self.startLat = startLat
-        self.startLon = startLon
-        self.startLev = start_level
-        self.map_widget.level = start_level
-
         self.mouse_x = None
         self.mouse_y = None
 
@@ -134,23 +124,16 @@ class MapView(QGraphicsView):
         self.pressed = False
         self.disableMove = False
 
-        self.startHe = None
-        self.startWi = None
-
         # updated later
         self.view_width = self.width()
         self.view_height = self.height()
 
         # Set initial zoom level (change the values as needed)
-        initial_zoom_factor = 1
-        self.schema_zoom = 1
+        initial_zoom_factor = 1.0
+        self.schema_zoom = 1.0
 
         self.scale(initial_zoom_factor, initial_zoom_factor)
 
-        self.selTempDistance = 20
-
-        self.selectedItems = list()
-
     def mousePressEvent(self, event: QMouseEvent):
         """
 
@@ -160,12 +143,7 @@ class MapView(QGraphicsView):
         self.map_widget.mousePressEvent(event)
         self.pressed = True
         self.disableMove = False
-        b = event.button()
-        if b == Qt.RightButton:
-            if not self.inItem:
-                for item in self.selectedItems:
-                    item.deSelectItem()
-                self.selectedItems.clear()
+
         super().mousePressEvent(event)
 
     def mouseReleaseEvent(self, event: QMouseEvent):
@@ -177,6 +155,7 @@ class MapView(QGraphicsView):
         self.map_widget.mouseReleaseEvent(event)
         self.pressed = False
         self.disableMove = True
+        super().mouseReleaseEvent(event)
 
     def mouseDoubleClickEvent(self, event: QMouseEvent):
         """
@@ -185,6 +164,7 @@ class MapView(QGraphicsView):
         :return:
         """
         self.map_widget.mouseDoubleClickEvent(event)
+        super().mouseDoubleClickEvent(event)
 
     def mouseMoveEvent(self, event: QMouseEvent):
         """
@@ -396,11 +376,7 @@ class MapWidget(QWidget):
 
         self.editor: GridMapWidget = editor
 
-        self.view = MapView(self.diagram_scene,
-                            map_widget=self,
-                            start_level=start_level,
-                            startLat=startLat,
-                            startLon=startLon)
+        self.view = MapView(scene=self.diagram_scene, map_widget=self)
 
         self.view.setBackgroundBrush(Qt.transparent)
 
@@ -425,8 +401,8 @@ class MapWidget(QWidget):
         # Internal vars
         # -------------------------------------------------------------------------
         # remember the tile source object
-        self.tile_src: Tiles = tile_src
-        self.tile_src.setCallback(self.on_tile_available)
+        self._tile_src: Tiles = tile_src
+        self._tile_src.setCallback(self.on_tile_available)
 
         # the tile coordinates
         self.level: int = start_level
@@ -481,9 +457,39 @@ class MapWidget(QWidget):
         # do a "resize" after this function
         QTimer.singleShot(10, self.resizeEvent)
 
-        self.GotoLevelAndPosition(level=start_level,
-                                  longitude=startLon,
-                                  latitude=startLat)
+        self.GotoLevelAndPosition(level=6,
+                                  longitude=0,
+                                  latitude=40)
+
+    @property
+    def tile_src(self) -> Tiles:
+        """
+        Get the current tile source
+        :return: Tiles
+        """
+        return self._tile_src
+
+    @tile_src.setter
+    def tile_src(self, tile_src: Tiles):
+        """
+        Set the current tile source
+        :param tile_src: Tiles
+        """
+
+        level, longitude, latitude = self.get_level_and_position()
+
+        if tile_src.TilesetName != self._tile_src.TilesetName:  # avoid changing tilesets to themselves
+            self._tile_src: Tiles = tile_src
+            self._tile_src.setCallback(self.on_tile_available)
+
+            if self.GotoLevel(level):
+                self.GotoLevelAndPosition(level=level, longitude=longitude, latitude=latitude)
+                self.view.centerSchema()
+            else:
+                while abs(self.view.schema_zoom - 0.015625) > 0.00001:
+                    self.view.schema_zoom = self.view.schema_zoom / self.view.map_widget.zoom_factor
+                    self.view.scale(1.0 / self.view.map_widget.zoom_factor, 1.0 / self.view.map_widget.zoom_factor)
+                self.GotoLevelAndPosition(level=0, longitude=None, latitude=None)
 
     @property
     def max_level(self):
@@ -586,13 +592,13 @@ class MapWidget(QWidget):
         :param event:
         :return:
         """
-
+        super().mousePressEvent(event)
         click_x = event.x()
         click_y = event.y()
 
         # assume we aren't dragging
-        self.start_drag_x = None
-        self.start_drag_y = None
+        # self.start_drag_x = None
+        # self.start_drag_y = None
 
         b = event.button()
         if b == Qt.MouseButton.NoButton:
