@@ -43,7 +43,7 @@ from GridCalEngine.basic_structures import Logger
 from GridCalEngine.enumerations import DeviceType, SimulationTypes
 
 from GridCal.Gui.Diagrams.graphics_manager import GraphicsManager
-import GridCal.Gui.Visualization.palettes as palettes
+import GridCalEngine.Devices.Diagrams.palettes as palettes
 from GridCal.Gui.messages import yes_no_question
 from GridCal.Gui.object_model import ObjectsModel
 
@@ -63,7 +63,8 @@ class BaseDiagramWidget(QSplitter):
                  diagram: Union[SchematicDiagram, MapDiagram],
                  library_model: Union[MapLibraryModel, SchematicLibraryModel],
                  time_index: Union[None, int] = None,
-                 call_delete_db_element_func: Callable[[Union[GridMapWidget, SchematicWidget], ALL_DEV_TYPES], None] = None):
+                 call_delete_db_element_func: Callable[
+                     [Union[GridMapWidget, SchematicWidget], ALL_DEV_TYPES], None] = None):
         """
         Constructor
         :param circuit:
@@ -118,7 +119,7 @@ class BaseDiagramWidget(QSplitter):
         self.circuit: MultiCircuit = circuit
 
         # diagram to store the objects locations
-        self.diagram: SchematicDiagram = diagram
+        self.diagram: Union[SchematicDiagram, MapDiagram] = diagram
 
         # class to handle the relationships between widgets and API objects
         self.graphics_manager = GraphicsManager()
@@ -353,7 +354,8 @@ class BaseDiagramWidget(QSplitter):
                 # plot the profiles
                 plt.show()
 
-    def set_rate_to_profile(self, api_object: ALL_DEV_TYPES):
+    @staticmethod
+    def set_rate_to_profile(api_object: ALL_DEV_TYPES):
         """
 
         :param api_object:
@@ -368,7 +370,8 @@ class BaseDiagramWidget(QSplitter):
                 if ok:
                     api_object.rate_prof.fill(api_object.rate)
 
-    def set_active_status_to_profile(self, api_object: ALL_DEV_TYPES, override_question=False):
+    @staticmethod
+    def set_active_status_to_profile(api_object: ALL_DEV_TYPES, override_question=False):
         """
 
         :param api_object:
@@ -572,3 +575,25 @@ class BaseDiagramWidget(QSplitter):
 
         if location is not None:
             location.draw_labels = draw_labels
+
+    def set_size_constraints(self,
+                             use_flow_based_width: bool = False,
+                             min_branch_width: int = 5,
+                             max_branch_width=5,
+                             min_bus_width=20,
+                             max_bus_width=20):
+        """
+        Set the size constraints
+        :param use_flow_based_width:
+        :param min_branch_width:
+        :param max_branch_width:
+        :param min_bus_width:
+        :param max_bus_width:
+        """
+        self.diagram.set_size_constraints(
+            use_flow_based_width=use_flow_based_width,
+            min_branch_width=min_branch_width,
+            max_branch_width=max_branch_width,
+            min_bus_width=min_bus_width,
+            max_bus_width=max_bus_width
+        )
