@@ -23,17 +23,18 @@ from PySide6 import QtGui, QtWidgets, QtCore
 from matplotlib import pyplot as plt
 from pandas.plotting import register_matplotlib_converters
 
-import GridCalEngine.Devices as dev
-import GridCalEngine.Simulations as sim
-import GridCal.Gui.GuiFunctions as gf
-from GridCal.Gui.object_model import ObjectsModel
-import GridCal.Gui.Visualization.palettes as palettes
+import GridCalEngine.Devices.Diagrams.palettes as palettes
 from GridCalEngine.IO.file_system import get_create_gridcal_folder
 from GridCal.Gui.GeneralDialogues import (CheckListDialogue, StartEndSelectionDialogue, InputSearchDialogue,
                                           InputNumberDialogue)
 from GridCalEngine.Devices.types import ALL_DEV_TYPES
 from GridCalEngine.enumerations import SimulationTypes
+from GridCalEngine.Devices.Diagrams.schematic_diagram import SchematicDiagram
 
+import GridCalEngine.Devices as dev
+import GridCalEngine.Simulations as sim
+import GridCal.Gui.GuiFunctions as gf
+from GridCal.Gui.object_model import ObjectsModel
 from GridCal.Gui.Diagrams.SchematicWidget.schematic_widget import (SchematicWidget,
                                                                    BusGraphicItem,
                                                                    generate_schematic_diagram,
@@ -67,26 +68,39 @@ class DiagramsMain(CompiledArraysMain):
         # list of diagrams
         self.diagram_widgets_list: List[ALL_EDITORS] = list()
 
+        # flag to avoid circular updating of the display settings when changing diagrams
+        self._enable_setting_auto_upgrade = True
+
         # Declare the map
         palettes_list = [palettes.Colormaps.GridCal,
                          palettes.Colormaps.Green2Red,
                          palettes.Colormaps.Heatmap,
                          palettes.Colormaps.TSO]
         self.cmap_dict = {e.value: e for e in palettes_list}
+        self.cmap_index_dict = {pal: i for i, pal in enumerate(palettes_list)}
         self.ui.palette_comboBox.setModel(gf.get_list_model([e.value for e in palettes_list]))
 
         # map tile sources
-        self.tile_sources = {
-            'Blue Marble': BlueMarbleTiles(tiles_dir=os.path.join(get_create_gridcal_folder(), 'tiles', 'blue_marble')),
-            'Carto positron': CartoDbTiles(
+        self.tile_sources = [
+            BlueMarbleTiles(
+                name='Blue Marble',
+                tiles_dir=os.path.join(get_create_gridcal_folder(), 'tiles', 'blue_marble')
+            ),
+            CartoDbTiles(
+                name='Carto positron',
                 tiles_dir=os.path.join(get_create_gridcal_folder(), 'tiles', 'carto_db_positron'),
-                tile_servers=['http://basemaps.cartocdn.com/light_all/']),
-            'Carto dark matter': CartoDbTiles(
+                tile_servers=['http://basemaps.cartocdn.com/light_all/']
+            ),
+            CartoDbTiles(
+                name='Carto dark matter',
                 tiles_dir=os.path.join(get_create_gridcal_folder(), 'tiles', 'carto_db_dark_matter'),
-                tile_servers=["http://basemaps.cartocdn.com/dark_all/"])
-        }
-
-        self.ui.tile_provider_comboBox.setModel(gf.get_list_model(list(self.tile_sources.keys())))
+                tile_servers=["http://basemaps.cartocdn.com/dark_all/"]
+            )
+        ]
+        tile_names = [tile.TilesetName for tile in self.tile_sources]
+        self.tile_index_dict = {tile.TilesetName: i for i, tile in enumerate(self.tile_sources)}
+        self.tile_name_dict = {tile.TilesetName: tile for tile in self.tile_sources}
+        self.ui.tile_provider_comboBox.setModel(gf.get_list_model(tile_names))
         self.ui.tile_provider_comboBox.setCurrentIndex(0)
 
         # Automatic layout modes
@@ -156,6 +170,8 @@ class DiagramsMain(CompiledArraysMain):
 
         # combobox change
         self.ui.plt_style_comboBox.currentTextChanged.connect(self.plot_style_change)
+        self.ui.palette_comboBox.currentTextChanged.connect(self.set_diagrams_palette)
+        self.ui.tile_provider_comboBox.currentTextChanged.connect(self.set_diagrams_map_tile_provider)
 
         # sliders
         self.ui.diagram_step_slider.sliderReleased.connect(self.colour_diagrams)
@@ -166,6 +182,14 @@ class DiagramsMain(CompiledArraysMain):
         self.ui.explosion_factor_doubleSpinBox.valueChanged.connect(self.explosion_factor_change)
         self.ui.defaultBusVoltageSpinBox.valueChanged.connect(self.default_voltage_change)
 
+        self.ui.min_branch_size_spinBox.valueChanged.connect(self.set_diagrams_size_contraints)
+        self.ui.max_branch_size_spinBox.valueChanged.connect(self.set_diagrams_size_contraints)
+        self.ui.min_node_size_spinBox.valueChanged.connect(self.set_diagrams_size_contraints)
+        self.ui.max_node_size_spinBox.valueChanged.connect(self.set_diagrams_size_contraints)
+
+        # check boxes
+        self.ui.branch_width_based_on_flow_checkBox.clicked.connect(self.set_diagrams_size_contraints)
+
         # context menu
         self.ui.diagramsListView.customContextMenuRequested.connect(self.show_diagrams_context_menu)
 
@@ -334,8 +358,8 @@ class DiagramsMain(CompiledArraysMain):
         max_branch_width = self.ui.max_branch_size_spinBox.value()
         min_bus_width = self.ui.min_node_size_spinBox.value()
         max_bus_width = self.ui.max_node_size_spinBox.value()
-        cmap_text = self.ui.palette_comboBox.currentText()
 
+        cmap_text = self.ui.palette_comboBox.currentText()
         cmap = self.cmap_dict[cmap_text]
 
         buses = self.circuit.buses
@@ -425,13 +449,13 @@ class DiagramsMain(CompiledArraysMain):
                 return diagram.colour_results(buses=buses,
                                               branches=branches,
                                               hvdc_lines=hvdc_lines,
-                                              Sbus=results.Sbus[t_idx, :],
+                                              Sbus=results.Sbus[-1, :],
                                               bus_active=bus_active,
-                                              Sf=results.Sf[t_idx, :],
-                                              St=results.St[t_idx, :],
-                                              voltages=results.voltages[t_idx, :],
+                                              Sf=results.Sf[-1, :],
+                                              St=results.St[-1, :],
+                                              voltages=results.voltages[-1, :],
                                               types=results.bus_types,
-                                              loadings=np.abs(results.loading[t_idx, :]),
+                                              loadings=np.abs(results.loading[-1, :]),
                                               br_active=br_active,
                                               hvdc_Pf=None,
                                               hvdc_Pt=None,
@@ -810,6 +834,29 @@ class DiagramsMain(CompiledArraysMain):
         else:
             return None
 
+    def create_blank_schematic_diagram(self, name: str = "") -> SchematicWidget:
+        """
+        Create a new schematic widget
+        :param name: name of the new schematic
+        :return:
+        """
+        diagram = SchematicDiagram(name=name)
+
+        diagram_widget = SchematicWidget(circuit=self.circuit,
+                                         diagram=diagram,
+                                         default_bus_voltage=self.ui.defaultBusVoltageSpinBox.value(),
+                                         time_index=self.get_diagram_slider_index(),
+                                         prefer_node_breaker=False,
+                                         call_delete_db_element_func=self.call_delete_db_element)
+
+        diagram_widget.setStretchFactor(1, 10)
+        diagram_widget.center_nodes()
+        self.add_diagram_widget_and_diagram(diagram_widget=diagram_widget, diagram=diagram)
+        self.set_diagrams_list_view()
+        self.set_diagram_widget(diagram_widget)
+
+        return diagram_widget
+
     def redraw_current_diagram(self):
         """
         Redraw the currently selected diagram
@@ -1000,6 +1047,32 @@ class DiagramsMain(CompiledArraysMain):
                                                                 diagram=diagram)
                             self.set_diagrams_list_view()
 
+    def new_bus_branch_diagram_from_substation(self, substation: dev.Substation):
+        """
+        Add a bus-branch diagram of a particular selection of objects
+        """
+
+        buses = self.circuit.get_substation_buses(substation=substation)
+
+        if len(buses):
+            diagram = make_vecinity_diagram(circuit=self.circuit,
+                                            root_bus=buses[0],
+                                            max_level=2,
+                                            prog_func=None,
+                                            text_func=None)
+
+            diagram_widget = SchematicWidget(self.circuit,
+                                             diagram=diagram,
+                                             default_bus_voltage=self.ui.defaultBusVoltageSpinBox.value(),
+                                             time_index=self.get_diagram_slider_index(),
+                                             call_delete_db_element_func=self.call_delete_db_element)
+
+            self.add_diagram_widget_and_diagram(diagram_widget=diagram_widget, diagram=diagram)
+            self.set_diagrams_list_view()
+        else:
+            info_msg(text=f"No buses were found associated with the substation {substation.name}",
+                     title="New schematic from substation")
+
     def create_circuit_stored_diagrams(self):
         """
         Create as Widgets the diagrams stored in the circuit
@@ -1023,17 +1096,21 @@ class DiagramsMain(CompiledArraysMain):
 
             elif isinstance(diagram, dev.MapDiagram):
                 # select the tile source from the diagram, if not fund pick the one from the GUI
-                defualt_tile_source = self.tile_sources[self.ui.tile_provider_comboBox.currentText()]
-                tile_source = self.tile_sources.get(diagram.tile_source, defualt_tile_source)
+                defualt_tile_source = self.tile_name_dict[self.ui.tile_provider_comboBox.currentText()]
+                tile_source = self.tile_name_dict.get(diagram.tile_source, defualt_tile_source)
 
                 # create the map widget
-                map_widget = GridMapWidget(tile_src=tile_source,
-                                           start_level=diagram.start_level,
-                                           longitude=diagram.longitude,
-                                           latitude=diagram.latitude,
-                                           name=diagram.name,
-                                           circuit=self.circuit,
-                                           diagram=diagram)
+                map_widget = GridMapWidget(
+                    tile_src=tile_source,
+                    start_level=diagram.start_level,
+                    longitude=diagram.longitude,
+                    latitude=diagram.latitude,
+                    name=diagram.name,
+                    circuit=self.circuit,
+                    diagram=diagram,
+                    call_delete_db_element_func=self.call_delete_db_element,
+                    call_new_substation_diagram_func=self.new_bus_branch_diagram_from_substation
+                )
 
                 # map_widget.GotoLevelAndPosition(5, -15.41, 40.11)
                 self.diagram_widgets_list.append(map_widget)
@@ -1048,7 +1125,7 @@ class DiagramsMain(CompiledArraysMain):
         Adds a Map diagram
         """
         # select the tile source
-        tile_source = self.tile_sources[self.ui.tile_provider_comboBox.currentText()]
+        tile_source = self.tile_name_dict[self.ui.tile_provider_comboBox.currentText()]
 
         diagram = generate_map_diagram(substations=self.circuit.get_substations(),
                                        voltage_levels=self.circuit.get_voltage_levels(),
@@ -1076,7 +1153,8 @@ class DiagramsMain(CompiledArraysMain):
                                    name=diagram.name,
                                    circuit=self.circuit,
                                    diagram=diagram,
-                                   call_delete_db_element_func=self.call_delete_db_element)
+                                   call_delete_db_element_func=self.call_delete_db_element,
+                                   call_new_substation_diagram_func=self.new_bus_branch_diagram_from_substation)
 
         self.add_diagram_widget_and_diagram(diagram_widget=map_widget, diagram=diagram)
         self.set_diagrams_list_view()
@@ -1161,6 +1239,24 @@ class DiagramsMain(CompiledArraysMain):
         index = self.ui.diagramsListView.model().index(row, 0)
         self.ui.diagramsListView.setCurrentIndex(index)
 
+        # set the properties
+        self._enable_setting_auto_upgrade = False
+        self.ui.branch_width_based_on_flow_checkBox.setChecked(widget.diagram.use_flow_based_width)
+        self.ui.min_branch_size_spinBox.setValue(widget.diagram.min_branch_width)
+        self.ui.max_branch_size_spinBox.setValue(widget.diagram.max_branch_width)
+        self.ui.min_node_size_spinBox.setValue(widget.diagram.min_bus_width)
+        self.ui.max_node_size_spinBox.setValue(widget.diagram.max_bus_width)
+        self.ui.palette_comboBox.setCurrentIndex(self.cmap_index_dict.get(widget.diagram.palette, 0))
+
+        if isinstance(widget, GridMapWidget):
+            self.ui.tile_provider_comboBox.setEnabled(True)
+            self.ui.tile_provider_comboBox.setCurrentIndex(self.tile_index_dict[widget.map.tile_src.TilesetName])
+        else:
+            self.ui.tile_provider_comboBox.setEnabled(False)
+
+        self.ui.defaultBusVoltageSpinBox.setValue(widget.diagram.default_bus_voltage)
+        self._enable_setting_auto_upgrade = True
+
     def plot_style_change(self):
         """
         Change the style
@@ -1699,3 +1795,45 @@ class DiagramsMain(CompiledArraysMain):
             self.circuit.delete_element(obj=api_obj)
         except ValueError as e:
             print(e)
+
+    def set_diagrams_size_contraints(self):
+        """
+        Set the size constraints
+        """
+        if self._enable_setting_auto_upgrade:
+            diagram_widget = self.get_selected_diagram_widget()
+
+            if diagram_widget is not None:
+                diagram_widget.set_size_constraints(
+                    use_flow_based_width=self.ui.branch_width_based_on_flow_checkBox.isChecked(),
+                    min_branch_width=self.ui.min_branch_size_spinBox.value(),
+                    max_branch_width=self.ui.max_branch_size_spinBox.value(),
+                    min_bus_width=self.ui.min_node_size_spinBox.value(),
+                    max_bus_width=self.ui.max_node_size_spinBox.value()
+                )
+                diagram_widget.diagram.default_bus_voltage = self.ui.defaultBusVoltageSpinBox.value()
+
+    def set_diagrams_palette(self):
+        """
+        Set the size constraints
+        """
+        if self._enable_setting_auto_upgrade:
+            diagram_widget = self.get_selected_diagram_widget()
+
+            if diagram_widget is not None:
+                cmap_text = self.ui.palette_comboBox.currentText()
+                cmap = self.cmap_dict[cmap_text]
+                diagram_widget.diagram.palette = cmap
+
+    def set_diagrams_map_tile_provider(self):
+        """
+        Set the size constraints
+        """
+        if self._enable_setting_auto_upgrade:
+            diagram_widget = self.get_selected_diagram_widget()
+
+            if diagram_widget is not None:
+                if isinstance(diagram_widget, GridMapWidget):
+                    tile_name = self.ui.tile_provider_comboBox.currentText()
+                    tile_src = self.tile_name_dict[tile_name]
+                    diagram_widget.map.tile_src = tile_src
\ No newline at end of file
