@@ -21,6 +21,7 @@ from matplotlib import pyplot as plt
 from matplotlib.colors import LinearSegmentedColormap
 
 from GridCal.Gui.associations_model import AssociationsModel
+from GridCal.Gui.table_view_header_wrap import HeaderViewWithWordWrap
 from GridCalEngine.DataStructures.numerical_circuit import NumericalCircuit, compile_numerical_circuit_at
 import GridCalEngine.basic_structures as bs
 import GridCalEngine.Devices as dev
@@ -87,6 +88,11 @@ class ObjectsTableMain(DiagramsMain):
         # Set context menu policy to CustomContextMenu
         self.ui.dataStructureTableView.setContextMenuPolicy(QtGui.Qt.ContextMenuPolicy.CustomContextMenu)
 
+        # wrap headers
+        self.ui.dataStructureTableView.setHorizontalHeader(HeaderViewWithWordWrap())
+        self.ui.profiles_tableView.setHorizontalHeader(HeaderViewWithWordWrap())
+        self.ui.associationsTableView.setHorizontalHeader(HeaderViewWithWordWrap())
+
         # combobox change
         self.ui.associationsComboBox.currentTextChanged.connect(self.display_associations)
 
@@ -359,18 +365,19 @@ class ObjectsTableMain(DiagramsMain):
         :return:
         """
         dev_type_text = self.get_db_object_selected_type()
+        model = self.get_current_objects_model_view()
+        association_prperty_name = self.ui.associationsComboBox.currentText()
 
-        if dev_type_text is not None:
+        if dev_type_text is not None and model is not None and association_prperty_name != "":
 
-            elements = self.get_current_objects_model_view().objects
+            elements = model.objects
 
-            association_prperty_name = self.ui.associationsComboBox.currentText()
-
-            if len(elements) > 0 and association_prperty_name != "":
+            if len(elements) > 0:
 
                 gc_prop = elements[0].get_property_by_name(prop_name=association_prperty_name)
                 associations: dev.Associations = elements[0].get_snapshot_value_by_name(name=association_prperty_name)
                 associated_objects = self.circuit.get_elements_by_type(device_type=associations.device_type)
+                self.ui.association_units_label.setText(gc_prop.units)
 
                 if len(associated_objects) > 0:
                     mdl = AssociationsModel(objects=elements,
@@ -383,8 +390,10 @@ class ObjectsTableMain(DiagramsMain):
                     self.ui.associationsTableView.setModel(None)
             else:
                 self.ui.associationsTableView.setModel(None)
+                self.ui.association_units_label.setText("")
         else:
             self.ui.associationsTableView.setModel(None)
+            self.ui.association_units_label.setText("")
 
     def display_objects_filter(self, elements: List[ALL_DEV_TYPES]):
         """
@@ -457,17 +466,20 @@ class ObjectsTableMain(DiagramsMain):
                 # update the associations view
                 assoc_mdl = gf.get_list_model(self.circuit.device_associations[elm_type])
                 self.ui.associationsComboBox.setModel(assoc_mdl)
+                self.display_associations()
 
             else:
                 self.ui.dataStructureTableView.setModel(None)
                 self.ui.device_type_magnitude_comboBox.clear()
                 self.ui.device_type_magnitude_comboBox_2.clear()
                 self.ui.associationsComboBox.clear()
+                self.ui.dataStructureTableView.setModel(None)
         else:
             self.ui.dataStructureTableView.setModel(None)
             self.ui.device_type_magnitude_comboBox.clear()
             self.ui.device_type_magnitude_comboBox_2.clear()
             self.ui.associationsComboBox.clear()
+            self.ui.dataStructureTableView.setModel(None)
 
     def get_selected_objects(self) -> List[ALL_DEV_TYPES]:
         """
