@@ -46,7 +46,7 @@ def config_data_to_struct(data_: Dict[str, Union[Dict[str, Any], str, Any]],
                 config_data_to_struct(corresponding_data, object_to_set)
 
             elif isinstance(object_to_set, QtWidgets.QComboBox):
-                index = object_to_set.findText(corresponding_data)
+                index = object_to_set.findText(str(corresponding_data))
                 if -1 < index < object_to_set.count():
                     object_to_set.setCurrentIndex(index)
 
@@ -114,9 +114,7 @@ class ConfigurationMain(ResultsMain):
                                    additional_qss="QToolTip {color: white; background-color: black; border: 0px; }")
 
             # note: The 0px border on the tooltips allow it to render properly
-
-            diagram = self.get_selected_diagram_widget()
-            if diagram is not None:
+            for diagram in self.diagram_widgets_list:
                 if isinstance(diagram, SchematicWidget):
                     diagram.set_dark_mode()
 
@@ -131,9 +129,7 @@ class ConfigurationMain(ResultsMain):
                                    additional_qss="QToolTip {color: black; background-color: white; border: 0px;}")
 
             # note: The 0px border on the tooltips allow it to render properly
-
-            diagram = self.get_selected_diagram_widget()
-            if diagram is not None:
+            for diagram in self.diagram_widgets_list:
                 if isinstance(diagram, SchematicWidget):
                     diagram.set_light_mode()
 
@@ -181,11 +177,11 @@ class ConfigurationMain(ResultsMain):
             "graphics": {
                 "dark_mode": self.ui.dark_mode_checkBox,
                 "palette": self.ui.palette_comboBox,
-                "min_node_size": self.ui.min_node_size_spinBox,
-                "max_node_size": self.ui.max_node_size_spinBox,
-                "min_branch_size": self.ui.min_branch_size_spinBox,
-                "max_branch_size": self.ui.max_branch_size_spinBox,
-                "width_based_flow": self.ui.branch_width_based_on_flow_checkBox,
+                # "min_node_size": self.ui.min_node_size_spinBox,
+                # "max_node_size": self.ui.max_node_size_spinBox,
+                # "min_branch_size": self.ui.min_branch_size_spinBox,
+                # "max_branch_size": self.ui.max_branch_size_spinBox,
+                # "width_based_flow": self.ui.branch_width_based_on_flow_checkBox,
                 "map_tile_provider": self.ui.tile_provider_comboBox,
                 "plotting_style": self.ui.plt_style_comboBox,
                 "video_fps": self.ui.fps_spinBox
@@ -407,8 +403,6 @@ class ConfigurationMain(ResultsMain):
                     self.save_gui_config()
                     print("GUI config file was erroneous, wrote a new one")
 
-
-
     def select_cgmes_boundary_set(self):
         """
         Select the current boundary set
