@@ -36,11 +36,13 @@ from GridCalEngine.Devices.multi_circuit import MultiCircuit
 import GridCalEngine.Simulations as sim
 from GridCalEngine.enumerations import EngineType
 from GridCalEngine.DataStructures.numerical_circuit import NumericalCircuit, compile_numerical_circuit_at
-import GridCal.Gui.GuiFunctions as gf
-import GridCal.Session.synchronization_driver as syncdrv
+
 from GridCalEngine.Compilers.circuit_to_bentayga import BENTAYGA_AVAILABLE
 from GridCalEngine.Compilers.circuit_to_newton_pa import NEWTON_PA_AVAILABLE
 from GridCalEngine.Compilers.circuit_to_pgm import PGM_AVAILABLE
+
+import GridCal.Gui.GuiFunctions as gf
+import GridCal.Session.synchronization_driver as syncdrv
 from GridCal.Gui.AboutDialogue.about_dialogue import AboutDialogueGuiGUI
 from GridCal.Gui.Analysis.AnalysisDialogue import GridAnalysisGUI
 from GridCal.Gui.ContingencyPlanner.contingency_planner_dialogue import ContingencyPlannerGUI
@@ -48,6 +50,7 @@ from GridCal.Gui.CoordinatesInput.coordinates_dialogue import CoordinatesInputGU
 from GridCal.Gui.GeneralDialogues import CheckListDialogue, StartEndSelectionDialogue
 from GridCal.Gui.messages import yes_no_question, warning_msg, info_msg, error_msg
 from GridCal.Gui.GridGenerator.grid_generator_dialogue import GridGeneratorGUI
+from GridCal.Gui.LoadCatalogue.catalogue_dialogue import CatalogueGUI
 from GridCal.Gui.Main.MainWindow import Ui_mainWindow, QMainWindow
 from GridCal.Gui.Main.object_select_window import ObjectSelectWindow
 from GridCal.Gui.ProfilesInput.models_dialogue import ModelsInputGUI
@@ -59,8 +62,6 @@ from GridCal.Gui.TowerBuilder.LineBuilderDialogue import TowerBuilderGUI
 from GridCal.Gui.GeneralDialogues import clear_qt_layout
 from GridCal.Gui.ConsoleWidget import ConsoleWidget
 from GridCal.Gui.Diagrams.generic_graphics import IS_DARK
-from GridCal.templates import (get_cables_catalogue, get_transformer_catalogue, get_wires_catalogue,
-                               get_sequence_lines_catalogue)
 
 
 def terminate_thread(thread):
@@ -167,6 +168,7 @@ class BaseMainGui(QMainWindow):
         self.file_sync_window: Union[SyncDialogueWindow, None] = None
         self.sigma_dialogue: Union[SigmaAnalysisGUI, None] = None
         self.grid_generator_dialogue: Union[GridGeneratorGUI, None] = None
+        self.catalogue_dialogue: Union[CatalogueGUI, None] = None
         self.contingency_planner_dialogue: Union[ContingencyPlannerGUI, None] = None
         self.analysis_dialogue: Union[GridAnalysisGUI, None] = None
         self.profile_input_dialogue: Union[ProfileInputGUI, None] = None
@@ -212,7 +214,7 @@ class BaseMainGui(QMainWindow):
         self.ui.actionLaunch_data_analysis_tool.triggered.connect(self.display_grid_analysis)
         self.ui.actionOnline_documentation.triggered.connect(self.show_online_docs)
         self.ui.actionReport_a_bug.triggered.connect(self.report_a_bug)
-        self.ui.actionAdd_default_catalogue.triggered.connect(self.add_default_catalogue)
+
         self.ui.actionFix_generators_active_based_on_the_power.triggered.connect(
             self.fix_generators_active_based_on_the_power
         )
@@ -228,6 +230,22 @@ class BaseMainGui(QMainWindow):
 
         self.ui.grid_name_line_edit.textChanged.connect(self.change_circuit_name)
 
+    def add_data_to_circuit(self, filename):
+        df = self.catalogue_dialogue.read_csv_file(filename)
+        print(df.head())
+
+        if self.catalogue_dialogue.ui.checkBox.isChecked():
+            # or self.catalogue_dialogue.ui.checkBox_5.isChecked()
+            transformers = self.catalogue_dialogue.get_transformer_catalogue(df)
+            self.circuit.transformer_types += transformers
+
+        elif self.catalogue_dialogue.ui.checkBox_2.isChecked():
+            self.circuit.underground_cable_types += self.catalogue_dialogue.get_cables_catalogue(df)
+
+        elif self.catalogue_dialogue.ui.checkBox_3.isChecked():
+            # or self.catalogue_dialogue.ui.checkBox_6.isChecked()
+            self.circuit.sequence_line_types += self.catalogue_dialogue.get_sequence_lines_catalogue(df)
+
     def LOCK(self, val: bool = True) -> None:
         """
         Lock the interface to prevent new simulation launches
@@ -743,16 +761,6 @@ class BaseMainGui(QMainWindow):
         """
         self.circuit.name = self.ui.grid_name_line_edit.text().strip()
 
-    def add_default_catalogue(self) -> None:
-        """
-        Add default catalogue to circuit
-        """
-
-        self.circuit.transformer_types += get_transformer_catalogue()
-        self.circuit.underground_cable_types += get_cables_catalogue()
-        self.circuit.wire_types += get_wires_catalogue()
-        self.circuit.sequence_line_types += get_sequence_lines_catalogue()
-
     def get_snapshot_circuit(self):
         """
         Get a snapshot compilation
